-----------------------------------------------------------------
-- Title: Two-way substitution box                             --
-- Filename: sbox.vhd                                          --
-----------------------------------------------------------------
-- Ver 1.0                                                     --
-- November 2008                                               --
-----------------------------------------------------------------
-- Author:                                                     --
-- Fischer V.                                                  --
-- (C) 2008 Laboratoire Hubert Curien                          --
-- France                                                      --
-----------------------------------------------------------------
-- Description :                                               --
--                                                             --
-- Sboxes are implemented in a dual-port synchronous embedded  --
-- memory organized in 8 x 256 bits, so that two substitutions --
-- can be realized using one memory block                      --
-- (one port per function).                                    --
--                                                             --
-- Inputs :                                                    --
--    * sboxa_in   : 8-bit input data of sbox A                --
--    * sboxb_in   : 8-bit input data of sbox B                --
--                                                             --
-- Outputs :                                                   --
--    * sboxa_out  : 8-bit output data of sbox A               --
--    * sboxb_out  : 8-bit output data of sbox B               --
-----------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.all;

ENTITY sbox IS
  PORT (
    clk         : IN std_logic;
    sboxa_in    : IN std_logic_vector (7 DOWNTO 0);
    sboxb_in    : IN std_logic_vector (7 DOWNTO 0);
    sboxa_out   : OUT std_logic_vector (7 DOWNTO 0);
    sboxb_out   : OUT std_logic_vector (7 DOWNTO 0)
     );
END sbox;

-------------- A R C H I T E C T U R E ------------------

ARCHITECTURE rtl OF sbox IS
-- True double port memory block
  COMPONENT tdp_mem8x256
    PORT (
      address_a : IN STD_LOGIC_VECTOR (7 DOWNTO 0);
      address_b : IN STD_LOGIC_VECTOR (7 DOWNTO 0);
      clock     : IN STD_LOGIC ;
      q_a       : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
      q_b       : OUT STD_LOGIC_VECTOR (7 DOWNTO 0)
    );
  END COMPONENT;

BEGIN
  
tdsbox_inst: tdp_mem8x256
  PORT MAP (
    address_a => sboxa_in,
    address_b => sboxb_in,
    clock     => clk,
    q_a       => sboxa_out,
    q_b       => sboxb_out
    );
END rtl;
