-----------------------------------------------------------------
-- Title: Cryptographic post-processing testbench              --
--                                                             --
-- Filename: tb_aes_enc128                                      --
-----------------------------------------------------------------
-- Ver 1.0                                                     --
-- January 2017                                                --
-----------------------------------------------------------------
-- Author:                                                     --
--   V. Fischer, SES, LaHC, Saint-Etienne, France              --
-----------------------------------------------------------------
-- Description :                                               --
--                                                             --
--                                                             --     
-----------------------------------------------------------------                                           
-- DUT: tb_aes_enc128.vhd, version 1.0                          --
-----------------------------------------------------------------                                          
-- Revision history:                                           --
--       - ...                                                 --
-----------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.numeric_std.all;
USE ieee.std_logic_unsigned.all;

USE ieee.std_logic_textio.all;
USE std.textio.all;

LIBRARY work;
USE work.txt_util.all;

LIBRARY work;
USE work.aes_pkg.ALL;

ENTITY tb_aes_enc128 IS
END tb_aes_enc128;


-------------- A R C H I T E C T U R E ------------------

ARCHITECTURE tb OF tb_aes_enc128 IS
 
  CONSTANT FAMILY         : string := FAMILY_VER;

  CONSTANT CLK_PER        : time:= 20 ns; -- 50 MHz
  
  SIGNAL clk_in           : STD_LOGIC := '0';          -- Input clock
  SIGNAL rst_in           : STD_LOGIC := '1';          -- Main reset (active high)
 
  SIGNAL start            : STD_LOGIC := '0';          -- Start encryption 
  SIGNAL busy             : STD_LOGIC := '0';          -- AES busy flag 

  SIGNAL ser_in           : STD_LOGIC := '0';          -- Input serial data
  
  SIGNAL data_in          : STD_LOGIC_VECTOR(127 DOWNTO 0) := X"00112233445566778899aabbccddeeff";  
                                                              -- Plain-Text input reg
  SIGNAL key_in           : STD_LOGIC_VECTOR(127 DOWNTO 0) := X"000102030405060708090a0b0c0d0e0f";  
                                                              -- User key input reg
  SIGNAL data_out         : STD_LOGIC_VECTOR(127 DOWNTO 0) := (OTHERS => '0');  
                                                              -- Cipher-Text output reg
  
 -- Format VHDL'93  --
  
-- DUT
  COMPONENT aes_enc128 IS
  GENERIC(
	  FAMILY   : STRING  := FAMILY_VER
    );
  PORT (
    clk         : IN STD_LOGIC;
    reset       : IN STD_LOGIC;
--    start       : IN STD_LOGIC;
    ser_in      : IN STD_LOGIC;
    busy        : OUT STD_LOGIC;
    data_out    : OUT STD_LOGIC_VECTOR (127 DOWNTO 0)
    );
  END COMPONENT;

------------------ B E G I N ------------------ 

BEGIN
-- Clock generator
proc_clk_in: PROCESS
  BEGIN
    WAIT FOR CLK_PER / 2; 
    clk_in <= NOT clk_in; -- Invertion of the clk signal
  END PROCESS;
 
DUT: aes_enc128
  GENERIC MAP(
    FAMILY   => FAMILY_VER
             )
  PORT MAP (
    clk      => clk_in,
	  reset    => rst_in,
--	  start    => start,
  	
	  ser_in   => ser_in,
--	  key_in   => key_in,

	  busy     => busy,
	  data_out => data_out    
	 );
  
 --------------------------------------------------------------------------
PROCESS 
  BEGIN      
	  rst_in   <= '1'; -- asserting RESET
    print2("Info from simulator: Reset asserted!");

    WAIT FOR CLK_PER *4;
    rst_in <= '0'; -- releasing RESET
	  print2("Info from simulator: Reset released!");
--	  WAIT FOR CLK_PER *2;
--	  WAIT UNTIL (clk_in = '1');
--    WAIT UNTIL (busy = '0');   
    FOR i IN 0 TO 127 LOOP
      ser_in <= data_in(i);
      WAIT UNTIL rising_edge(clk_in);
    END LOOP;
    FOR i IN 0 TO 127 LOOP
      ser_in <= key_in(i);
      WAIT UNTIL rising_edge(clk_in);
    END LOOP;
    
    -- print2("Master Key: writting MK word!");
		-- key_in  <= X"000102030405060708090a0b0c0d0e0f" AFTER 1 ns;
    -- data_in <= X"00112233445566778899aabbccddeeff" AFTER 1 ns;
	  WAIT FOR CLK_PER*5000;
	  WAIT UNTIL (clk_in = '1');
    start   <= '1' AFTER 1 ns;
	  WAIT UNTIL (clk_in = '1');
    start   <= '0' AFTER 1 ns;    
	  WAIT FOR CLK_PER * 23;
	 
    WAIT;

  END PROCESS;
	 

END tb;